%==========================================================================
% Step 01: this file reads WIOD data and computes the required
% cubes Yi3D, Yis3D, Xijs3D, Lijs3D (lambdas), Dj3d (trade deficits) and
% betas AND (!!!) aggregates sectors and countries (see lines 36-46)
%==========================================================================
clear all;	% MODIFIED VERSION OF STEP_01_READ_WIOD_2009: CORRECT FOR NEGATIVE ENTRIES IN ONVENTORIES AND GROSS FIXED CAPITAL FORMATION
N=41;   S=35;	% number of regions and sectors


%======================= Read the data from csv-file ======================
DATA=dlmread('WIOT08_ROW_Apr12.csv',',');
Zinit=DATA(1:1435,1:1435);	% extract data on intermediate input flows
X=DATA(1:1435,1:1640);	% extract data on both flows of intemediate and final goods
Rinit=sum(X,2);


%====================== Make the required corrections =====================
%... Adjustment #1: think about IO model and data as X=A*X+F+INV, where INV
% represents a vector of NEGATIVE changes in inventories (positive changes
% are a part of final demand vector, F. Now we have X=inv(I-A)*(F+INV). We
% need to convert reduction in inventories into increased output (i.e. we 
% can think about this exercise as the one that expands the time period, so 
% that now we also take the previous period production of these inventories,
% that are reduced, into account). Keeping F the same as in data, we set 
% INV=0 and recompute vector X and matrix of flows F

FIN=X(1:1435, 1436:1640);  % matrix of final demand and inventories adjustment
FINsum=sum(FIN,2);  % initial vector of final demand including negative INV
F=FIN.*(FIN>0); % positive component of final demand (becomes final demand after correcting for INV<0)
Fsum=sum(F,2); % initial final demand (remains the same)
A=Zinit/diag(Rinit+.000001.*(Rinit<=0.000001));    % matrix on direct input coefficients (small number is added to avoid NaN for zero-sectors)
R=(eye(N*S)-A)\(Fsum);  % compute a new vector of total output under zero decline in inventories
Z=A*diag(R);            % compute a new matrix of intermediate goods flows    


%====================== Aggregate sectors & countries =====================
%... For the aggregation scheme see the "AGG_MATRICES.ods" file.
% Loading matrices for aggregating sectors and countries
AggC=dlmread('AGG_C_10.csv');  N=size(AggC,1);
AggS=dlmread('AGG_S_16.csv');  S=size(AggS,1);
CC=kron(AggC,AggS); % matrix for aggregating matrix of intermediate goods flows Z
FF=kron(AggC,eye(5));  % matrix for aggregating matrix of final demand F
Z=(CC) * (Z) * (CC');   % aggregated Z matrix
F=(CC) * (F) * (FF');   % aggregated F matrix
X=[Z,F];
R=sum(X,2);


%======================= Compute the required data ========================
ls=ones(S,1);	ln=ones(N,1);	l5=ones(5,1);   % summer vectors
AUX1=kron(eye(N),ls);	% aux matrix for summation over "k": ijsk -> ijs
AUX2=kron(eye(N),l5);
AUX=[AUX1;AUX2];
Xijs3D=permute(reshape(X*AUX,S,N,N),[2 3 1]); % Xijs -- flow of good "s" from "i" to "j"
Xjs3D=repmat(sum(Xijs3D,1),[N 1 1]);    % Xjs -- total purchases of good "s" by "j"   
Xj3D=repmat(sum(Xjs3D,3),[1 1 S]);  % Xj -- total expenditures of country "j"
Lijs3D=Xijs3D./Xjs3D;	% "cube" w/ elem(i,j,s)=lambda_ijs, share of expenditures

Ris3D=permute(repmat(reshape(sum(X,2),S,N),[1 1 N]),[2 3 1]);	% "cube" w/ elem(i,j,s)=R_is
Rjs3D=permute(Ris3D,[2 1 3]);   % "cube" w/ elem(i,j,s)=R_js
Ri3D=repmat(sum(Ris3D,3),[1 1 S]);	% "cube" w/ elem(i,j,s)=R_i, total output(!)
Rj3D=permute(Ri3D,[2 1 3]);

Djs3D=Xjs3D-Rjs3D;  % vector of trade deficit of country "j" in good "s" (D>0 means deficit)
Dj3D=repmat(sum(Djs3D,3),[1 1 S]);	% "cube" w/ elem(i,j,s)=D_j

l_alpha=repmat(eye(S),[1 N]);
Zsjk3D=permute(reshape((l_alpha*Z),S,S,N),[2 3 1]); % (s,j,k): flows of inputs from sector "k" to "s" in country "j" ("k" comes from all over the world)
Rsjk3D=repmat((reshape(Rjs3D(1,:,:),N,S))',[1 1 S]); % (s,j,k): total output of sector "j", Rjs
alpha_tilde_sjk3D=Zsjk3D./(Rsjk3D+0.00000001*(Rsjk3D==0)); % add small value to avoid NaN in alphas
alpha_tilde_sj3D=repmat(sum(alpha_tilde_sjk3D,3),[1 1 S]);
alpha_tilde_sj2D=alpha_tilde_sj3D(:,:,1);

% Adjust alphas as  (alpha_new_j,ks=alpha_j,s*e_j,k) -- BHR approach
ejs3D=Xjs3D./Xj3D;  % e_j,s in (i,j,s)
ejk_sjk3D=repmat(ejs3D(1,:,:),[S 1 1]);  %e_j,k in (s,j,k)
alpha_BHR_sjk3D=alpha_tilde_sj3D.*ejk_sjk3D;
alpha_BHR_sj3D=repmat(sum(alpha_BHR_sjk3D,3),[1 1 S]);

AUX0=kron(eye(N),l5);
Fijs3D=permute(reshape(F*AUX0,S,N,N),[2 3 1]); % final consumption of good "s" in country "j" that comes from "i"
Fjs3D=repmat(sum(Fijs3D,1),[N 1 1]);    % total final consumption of "s" in "j"
Fj3D=repmat(sum(Fjs3D,3),[1 1 S]);  % total final consumption in "j"
betajs3D=Fjs3D./Fj3D;   % compute betas as shares of consumption

% for checking if F=GDP+deficit
VAis=(R')-sum(Z,1);  VAis=(VAis>0).*VAis;    % ... to remove small values<0
Yis3D=permute(repmat(reshape(VAis,S,N),[1 1 N]),[2 3 1]);  % Yis -- GDP (or VA) in country "i" and sector "s"
Yi3D=repmat(sum(Yis3D,3),[1 1 S]);
Yj3D=permute(Yi3D,[2 1 3]);

epsilon=dlmread('SECTORAL_TRADE_ELASTICITY.csv');	
epsilon_s=epsilon(:,2); epsilon_s([5,18:length(epsilon)])=[];    % drop elasticities for aggregated sectors: for initial sectors 5 and 4 elasticities are the same, while initial sectors 17:35 are non-tradable (also have the same elasticity)
epsilon_s(end,1)=5;   % set trade elasticities for non-tradables to 5
epsilon_s3D=reshape(kron(epsilon_s',ones(N)),N,N,S);

% save the data as a mat-file for models with MC and intermed. goods (where betas are share of consumption and Y is GDP)
save Step_02_WIOD_AGG_10x16_with_INT.mat N S Xijs3D Xjs3D Lijs3D Yis3D Yi3D Yj3D Ris3D Ri3D Rj3D Fijs3D Fjs3D Fj3D Dj3D betajs3D alpha_tilde_sjk3D alpha_tilde_sj3D alpha_BHR_sjk3D alpha_BHR_sj3D epsilon_s3D;% alpha_tilde_diag_sjjk3D;  % the last "cube" is constructed for diag_IO_GT

% save the data as a mat-file for models with PC and no intermed. goods (where betas are share of total expenditures and Y is total output)
consjs3D=betajs3D;
betajs3D=Xjs3D./Xj3D;   % shares of total expenditures 
VAis3D=Yis3D;   VAi3D=Yi3D; VAj3D=Yj3D;
Yis3D=Ris3D;    Yi3D=Ri3D;  Yj3D=Rj3D;
sigma_s3D=1+epsilon_s3D;
save Step_02_WIOD_AGG_10x16_no_INT.mat N S Xijs3D Xjs3D Lijs3D VAis3D VAi3D VAj3D Yis3D Yi3D Yj3D Dj3D betajs3D consjs3D epsilon_s3D sigma_s3D;
